﻿
using System;
using System.Collections.Generic;
using System.Threading;
namespace HeuristicLab.Clients.Hive.SlaveCore {

  public class ExecutorQueue : MarshalByRefObject {
    private Queue<ExecutorMessage> queue = null;
    private Semaphore semaphore = null;

    /// <summary>
    /// Creates a new MessageQueue Object.
    /// A new Queue and a Semaphore is created. The Semaphore is set to a max size of 5000.
    /// </summary>
    public ExecutorQueue() {
      queue = new Queue<ExecutorMessage>();
      semaphore = new Semaphore(0, 5000);
    }

    /// <summary>
    /// Returns the oldest ExecutorMessage Object from the Queue. 
    /// </summary>
    /// <returns>the oldest ExecutorMessage Object</returns>
    public ExecutorMessage GetMessage() {
      semaphore.WaitOne();
      lock (this) {
        if (queue.Count > 0) {
          return queue.Dequeue();
        }
      }
      return null;
    }

    /// <summary>
    /// Adds a ExecutorMessage Object to the Queue 
    /// </summary>
    /// <param name="message">the ExecutorMessage</param>
    public void AddMessage(ExecutorMessage message) {
      lock (this) {
        queue.Enqueue(message);
        semaphore.Release();
      }
    }

    /// <summary>
    /// Adds a message to the Queue. The ExecutorMessage Object is built in the Method
    /// </summary>
    /// <param name="message">the Message</param>
    public void AddMessage(ExecutorMessageType message) {
      lock (this) {
        queue.Enqueue(new ExecutorMessage(message));
        semaphore.Release();
      }
    }
  }
}
